package org.cds.vizmine;

import java.net.MalformedURLException;
import java.util.ArrayList;

/**
 * Class containing tools to calculate coordinate from Hammer & Aitoff projection, given (x, y) in the projection (x and y are cartesian coordinate system)
 * Also containing tools to convert into the Galactical Coordinate System.
 * @author Maxime Royer
 *
 */
public class Tools {
	
	/**
	 * Ratio which represents the reduction of the picture size.
	 */
	final static double RATIO = 243./320.;
	
	/**
	 * Constants to transform a angle from radian to degree
	 */
	final static double RADIANTODEGREE = 180 / Math.PI;
	
	/**
	 * Function which converts y into Hammer-Aitoff projection coordinate.
	 * @param x
	 * @param xmin
	 * @param xmax
	 * @param factor
	 * @return la distance au centre (sur Ox)
	 */
	public static double getXConverted(double x, double xmin, double xmax, double factor) {
		return ((xmax + xmin) / 2 - x) / factor;
	}
	
	/**
	 * Function which converts y into Hammer-Aitoff projection coordinate.
	 * @param y
	 * @param ymin
	 * @param ymax
	 * @param factor
	 * @return la distance au centre (sur Oy)
	 */
	public static double getYConverted(double y, double ymin, double ymax, double factor) {
		return ((ymax + ymin) / 2 - y) / factor;
	}
	
	/**
	 * Function which calculates z².
	 * @param x
	 * @param y
	 * @return z²
	 */
	public static double getZ(double x, double y) {
		return 1 - Math.pow((0.25 * x), 2) - Math.pow((0.5 * y), 2);
	}
	
	/**
	 * Function which calculates the longitude of the point (x, y).
	 * @param x
	 * @param y
	 * @return the longitude of the point (x, y) (in rad)
	 */
	public static double getLongitude(double x, double y) {
		return 2 * Math.atan((x * Math.sqrt(getZ(x, y))) / (2 * (2 * getZ(x, y) - 1))) + Math.PI;
	}
	
	/**
	 *  Function which calculates the longitude of the point (x, y).
	 * @param x
	 * @param y
	 * @return the longitude of the point (x, y) (in deg)
	 */
	public static double getLongitudeDegree(double x, double y) {
		return RADIANTODEGREE * getLongitude(x, y);
	}
	
	/**
	 * Function which calculates the latitude of the point (x, y).
	 * @param x
	 * @param y
	 * @return the latitude of the point (x, y) (in rad)
	 */
	public static double getLatitude(double x, double y) {
		return Math.asin(y * Math.sqrt(getZ(x, y)));
	}
	
	/**
	 * Function which calculates the latitude of the point (x, y).
	 * @param x
	 * @param y
	 * @return the latitude of the point (x, y) (in deg)
	 */
	public static double getLatitudeDegree(double x, double y) {
		return RADIANTODEGREE * getLatitude(x, y);
	}
	
	/**
	 * J2000 Matrix, useful for galactic coordinate transform.
	 * 
	 */
	public static double[][] J2000 = {
		{-0.054875539726,	-0.873437108010,	-0.483834985808	},
		{ 0.494109453312,	-0.444829589425,	 0.746982251810	},
		{-0.867666135858,	-0.198076386122,	 0.455983795705	}
	};
	
	/**
	 * Function which transform the longitude l and the latitude b into the galactic coordinate system.
	 * @param l
	 * @param b
	 * @return
	 */
	public static Galactic getGalacticTransform(double l, double b) {
		return new Galactic(l, b);
	}
	
	/**
	 * Function which retrieves the XML file generated by the user's clic.
	 * @param x
	 * @param y
	 * @param xmax
	 * @param ymax
	 * @param ratio
	 * @return
	 * @throws MalformedURLException
	 */
	public static ArrayList<String> getListOfCatalogues(double x, double y, double xmax, double ymax) throws MalformedURLException {
		return new RetrieveData("http://vizier.u-strasbg.fr/viz-bin/vizmine.exe/-vmine.limit=" + 5 * RATIO + "," + 5 * RATIO + ":" + xmax + "," + ymax + "/-c.u=arcmin/-c.r=15/-c.eq=G/-vmine.out=xml?"+x+","+y).parse();
	}
}
