package org.cds.vizmine;

import java.net.MalformedURLException;
import java.util.ArrayList;

import android.content.Context;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.AttributeSet;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.widget.TextView;

/**
 * Class extends SurfaceView, which contains images to draw on the background
 * and the foreground according to the user when he touches the screen. It shows
 * on screen the sky and a cursor to move, and below the coordinates
 * corresponding, with the catalogues available on the database, related to this
 * position.
 * 
 * @author Maxime Royer
 * 
 */
public class VizMineView extends SurfaceView implements SurfaceHolder.Callback {
	
	public static long IDLE_TIME = 1500;
	
	class VizMineThread extends Thread {

		private long mPreviousTime, mCurrentTime;
		private ArrayList<String> list;

		/**
		 * The drawable to use as the background of the animation canvas
		 */
		private Bitmap mBackgroundImage;

		/**
		 * Current height and width of the surface
		 */
		private int mCanvasHeight, mCanvasWidth;

		/**
		 * Message handler used by thread to interact with TextView
		 */
		private Handler mHandler;

		/**
		 * Handle to the surface manager object we interact with
		 */
		private SurfaceHolder mSurfaceHolder;

		/**
		 * Pixel height and width of cursor image
		 */
		final private int mCursorHeight, mCursorWidth;

		/**
		 * What to draw for the Cursor
		 */
		private Drawable mCursorImage;

		/**
		 * (mX, mY) represents the center of the cursor (mXc, mYx) represents
		 * the (mX, mY) converted coordinate
		 */
		private double 	mX,	mXConverted,
						mY, mYConverted;

		/**
		 * Ratio earth/background XMIN, XMAX, YMIN, YMAX are the bounds of the
		 * image
		 */
		final private double 	XMIN = 5 * Tools.RATIO, 	XMAX, 	XCENTRE,
								YMIN = 5 * Tools.RATIO,		YMAX,	YCENTRE,
								FACTOR;

		/**
		 * Indicate whether the surface has been created and id ready to draw
		 */
		private boolean mRun;

		public VizMineThread(SurfaceHolder surfaceHolder, Context context, Handler handler) {
			mSurfaceHolder = surfaceHolder;
			mContext = context;
			mHandler = handler;

			Resources res = mContext.getResources();

			mCursorImage = res.getDrawable(R.drawable.cursor);
			mBackgroundImage = BitmapFactory.decodeResource(res, R.drawable.vizmine480);

			mCursorWidth = mCursorImage.getIntrinsicWidth();
			mCursorHeight = mCursorImage.getIntrinsicHeight();

			XMAX = mBackgroundImage.getWidth() - XMIN;
			XCENTRE = XMAX / 2 + XMIN / 2;
			YMAX = (mBackgroundImage.getHeight() - YMIN) * Tools.RATIO;
			YCENTRE = YMAX / 2 + YMIN / 2;

			FACTOR = (YMAX - YMIN) / (2 * Math.sqrt(2));

			// init
			mX = XCENTRE;
			mY = YCENTRE;
			mXConverted = 0;
			mYConverted = 0;

			mCurrentTime = System.currentTimeMillis();
			mPreviousTime = mCurrentTime;
			mRun = true;
		}
		
		public boolean isIdle() {
			return mCurrentTime - mPreviousTime > IDLE_TIME;
		}

		@Override
		public void run() {
			while (mRun) {
				Canvas c = null;
				try {
					c = mSurfaceHolder.lockCanvas(null);
					synchronized (mSurfaceHolder) {
						update();
						doDraw(c);
						try {
							fillList();
						} catch (MalformedURLException e) {
							e.printStackTrace();
						}
					}
				} finally {
					if (c != null)
						mSurfaceHolder.unlockCanvasAndPost(c);
				}
				updateCurrentTime();
			}
		}

		public void setRunning() {
			mRun = true;
		}

		public void unSetRunning() {
			mRun = false;
		}

		public void updateCurrentTime() {
			synchronized (mSurfaceHolder) {
				mCurrentTime = System.currentTimeMillis();
			}
		}

		public void updatePreviousTime() {
			synchronized (mSurfaceHolder) {
				mPreviousTime = mCurrentTime;
			}
		}

		public long getPreviousTime() {
			return mPreviousTime;
		}
		
		public long getCurrentTime() {
			return mCurrentTime;
		}

		public void printState() {
			synchronized (mSurfaceHolder) {
				Message msg = mHandler.obtainMessage();
				Bundle b = new Bundle();

				b.putString("longlat", 		"l: " + (Math.floor(Tools.getLongitudeDegree(mXConverted, mYConverted) * 100)) / 100
										+ "\nb: " + (Math.floor(Tools.getLatitudeDegree(mXConverted, mYConverted) * 100) / 100));
				b.putInt("longlat_visible", View.VISIBLE);

				b.putString("radec", Tools.getGalacticTransform(Tools.getLongitudeDegree(mXConverted, mYConverted), Tools.getLatitudeDegree(mXConverted, mYConverted)).toString());
				b.putInt("radec_visible", View.VISIBLE);
				msg.setData(b);
				mHandler.sendMessage(msg);
			}
		}

		public void setSurfaceSize(int w, int h) {
			synchronized (mSurfaceHolder) {
				mCanvasWidth = w;
				mCanvasHeight = h;
				mBackgroundImage = Bitmap.createScaledBitmap(mBackgroundImage, mCanvasWidth, mCanvasHeight, true);
			}
		}

		/**
		 * Since this is a Hammer projection, we have to use the right
		 * conversion for this coordinate system. See {@link
		 * org.cds.vizmine.Tools.class}
		 * 
		 * @param event
		 * @return true if the surface has to be drawn, else false
		 */
		public boolean onTouchEvent(MotionEvent event) {
			synchronized (mSurfaceHolder) {
				if (! VizMine.isOpen) {
					updatePreviousTime();
					double 	x = event.getX(),
							y = event.getY();

					/* il faut normer les coordonnées (x, y) */
					double 	xc = Tools.getXConverted(x, XMIN, XMAX, FACTOR),
							yc = Tools.getYConverted(y, YMIN, YMAX, FACTOR);

					if (Tools.getZ(xc, yc) < 0.5)
						return false;

					/* (x, y) is IN the projection */
					mX = x;
					mY = y;
					mXConverted = xc;
					mYConverted = yc;
					updateCurrentTime();
				}
			}
			return true;
		}

		public boolean onTouch(View v, MotionEvent e) {
			return onTouchEvent(e);
		}

		public void doDraw(Canvas canvas) {
			canvas.drawBitmap(mBackgroundImage, 0, 0, null);
			canvas.save();

			int yTop = (int) mY - mCursorHeight / 2;
			int xLeft = (int) mX - mCursorWidth / 2;
			mCursorImage.setBounds(xLeft, yTop, xLeft + mCursorWidth, yTop + mCursorHeight);
			mCursorImage.draw(canvas);

			canvas.restore();
		}

		public void update() {
			if (mRun) {
				printState();
			}
		}

		public boolean doKeyUp(int keyCode, KeyEvent msg) {
			return doKeyDown(keyCode, msg);
		}

		public boolean doKeyDown(int keyCode, KeyEvent msg) {
			synchronized (mSurfaceHolder) {
				if (! VizMine.isOpen) {
					updatePreviousTime();
					if (keyCode == KeyEvent.KEYCODE_DPAD_UP) {
						if (Tools.getZ(mXConverted, mYConverted) > 0.5)
							mYConverted = Tools.getYConverted(--mY, YMIN, YMAX, FACTOR);
						return true;
					} else if (keyCode == KeyEvent.KEYCODE_DPAD_LEFT) {
						if (Tools.getZ(mXConverted, mYConverted) > 0.5)
							mXConverted = Tools.getXConverted(--mX, XMIN, XMAX, FACTOR);
						return true;
					} else if (keyCode == KeyEvent.KEYCODE_DPAD_DOWN) {
						if (Tools.getZ(mXConverted, mYConverted) > 0.5)
							mYConverted = Tools.getYConverted(++mY, YMIN, YMAX, FACTOR);
						return true;
					} else if (keyCode == KeyEvent.KEYCODE_DPAD_RIGHT) {
						if (Tools.getZ(mXConverted, mYConverted) > 0.5)
							mXConverted = Tools.getXConverted(++mX, XMIN, XMAX, FACTOR);
						return true;
					}
					updateCurrentTime();
				}
			}
			return false;
		}
		
		private void fillList() throws MalformedURLException {
			list = Tools.getListOfCatalogues(mX, mY, XMAX, YMAX);
		}
		
		public ArrayList<String> getList() {
			return list;
		}

		public Bundle saveState(Bundle map) {
			synchronized (mSurfaceHolder) {
                if (map != null) {
                    map.putDouble("mX", Double.valueOf(mX));
                    map.putDouble("mY", Double.valueOf(mY));
                    map.putDouble("mXConverted", Double.valueOf(mXConverted));
                    map.putDouble("mYConverted", Double.valueOf(mYConverted));
                    map.putStringArrayList("list", getList());
                    map.putLong("mPreviousTime", Long.valueOf(mPreviousTime));
                    map.putLong("mCurrentTime", Long.valueOf(mCurrentTime));
                }
			}		
			return map;
		}

		public synchronized void restoreState(Bundle restore) {
			synchronized(mSurfaceHolder) {
				mX = restore.getDouble("mX");
				mY = restore.getDouble("mY");
				mXConverted = restore.getDouble("mXConverted");
				mYConverted = restore.getDouble("mYConverted");
				list = restore.getStringArrayList("list");
				mPreviousTime = restore.getLong("mPreviousTime");
				mCurrentTime = restore.getLong("mCurrentTime");
			}
		}
	} // End of class VizMineThread

	private Context mContext;

	private TextView mLongLat;
	private TextView mRaDec;

	private VizMineThread thread;

	public VizMineView(Context context, AttributeSet attrs) {
		super(context, attrs);

		SurfaceHolder holder = getHolder();
		holder.addCallback(this);

		thread = new VizMineThread(holder, context, new Handler() {
			@Override
			public void handleMessage(Message m) {
				mLongLat.setVisibility(m.getData().getInt("longlat_visible"));
				mLongLat.setText(m.getData().getCharSequence("longlat"));

				mRaDec.setVisibility(m.getData().getInt("radec_visible"));
				mRaDec.setText(m.getData().getCharSequence("radec"));
			}
		});

		setFocusable(true);
	}

	public VizMineThread getThread() {
		return thread;
	}

	public boolean onTouchEvent(MotionEvent event) {
		return thread.onTouchEvent(event);
	}

	public boolean onKeyDown(int keyCode, KeyEvent msg) {
		return thread.doKeyDown(keyCode, msg);
	}

	public void setTextView(TextView tv1, TextView tv2) {
		mLongLat = tv1;
		mRaDec = tv2;
	}

	@Override
	public void surfaceChanged(SurfaceHolder holder, int format, int w, int h) {
		thread.setSurfaceSize(w, h);
	}

	@Override
	public void surfaceCreated(SurfaceHolder holder) {
		thread.start();
		thread.setRunning();
	}

	@Override
	public void surfaceDestroyed(SurfaceHolder holder) {
		boolean retry = true;
		thread.stop();
		thread.unSetRunning();
		while (retry) {
			try {
				thread.join();
				retry = false;
			} catch (InterruptedException e) {
			}
		}
	}
} // End of class VizMineView
